#!/usr/bin/env python
# -*- coding: iso-8859-15 -*-

import os
import cherrypy
from Cheetah.Template import Template
import time
import docutils.core
import gettext
from glob import glob

TEMPLATE_CODING = "iso-8859-15"
APPDIR = os.path.dirname(os.path.abspath(__file__))
INI_FILENAME = os.path.join(APPDIR, "myapp.ini")
LANGUAGE_CODES = ["de", "en"] # erster Eintrag ist Standard


class CheetahBaseTemplate(Template):
    """
    Cheetah-Vorlage mit zustzlichen Attributen wie z.B. rest2html zum Rendern
    von reStructuredText. Die Hauptvorlage wird von dieser Klasse abgeleitet.
    """
    
    def rest2html(
        self, input_string, input_encoding = 'iso-8859-15', 
        output_encoding = 'iso-8859-15', doctitle = 0, initial_header_level = 2,
        language_code = "de"
    ):
        """
        Given an input string, returns an HTML fragment as a string.
        
        The return value is the contents of the <body> tag, less the title,
        subtitle, and docinfo.
        """
        
        if callable(input_string):
            input_string = str(input_string())
        input_string = unicode(input_string, input_encoding, "replace")
        
        overrides = {
            "input_encoding": input_encoding,
            "doctitle_xform": doctitle,
            "initial_header_level": initial_header_level,
            "language_code": language_code,
        }
        parts = docutils.core.publish_parts(
            source = input_string, writer_name='html', settings_overrides = overrides
        )
        fragment = parts['fragment']
        if output_encoding != 'unicode':
            fragment = fragment.encode(output_encoding, "replace")
        
        return fragment
    
    
    def breadcrumb(self, language_code = "de"):
        """
        Gibt eine Liste aus URL/Titel-Tuppeln zurck. Diese wird verwendet um
        eine Breadcrumb-Leiste anzuzeigen.
        """
        
        retlist = []
        
        logical_path = [ 
            item for item in cherrypy.request.path_info.split("/") if item 
        ]
        
        while True:
            if not logical_path:
                break
            crumbpath = os.path.join(APPDIR, *logical_path)
            if os.path.isdir(crumbpath):
                for title_path in [
                    os.path.join(crumbpath, "%s.menutitle.meta" % language_code),
                    os.path.join(crumbpath, "menutitle.meta"),
                ]:
                    if os.path.isfile(title_path):
                        f = file(title_path, "rU")
                        title = f.readline().strip()
                        f.close()
                        if title:
                            url = "/" + "/".join(logical_path) + "/"
                            retlist.append((url, title))
                            break
            logical_path.pop()
        
        retlist.reverse()
        return retlist


class Root(object):
    
    def robots_txt(self):
        """
        Gibt die robots.txt zurck
        """
        
        # URLs sammeln und zusammensetzen
        disallowed_links = [
            "/images/",
            "/js/",
            "/css/",
        ]
        
        # robots.txt zusammensetzen
        robots_txt = (
            "# robots.txt\n"
            "\n"
            "User-agent: *\n"
            "#for $disallowed_link in $disallowed_links\n"
            "Disallow: $disallowed_link\n"
            "#end for\n"
            "\n"
        )
        robots_template = Template(robots_txt)
        robots_template.source_links = source_links
        
        # robots.txt zurck geben
        cherrypy.response.headers["Content-Type"] = "text/plain"
        return str(robots_template)
    
    robots_txt.exposed = True
    
    
    def default(self, *args, **kwargs):
        """
        Standard-Handler fr Objekte, die keiner anderen Handler-Funktion
        zugewiesen werden konnten.
        
        TMPL-Dateien werden hier auch ohne zugewiesener Funktion abgearbeitet
        und ausgeliefert.
        """
        
        args = list(args)
        
        # Language Code aus args herausholen
        language_code = LANGUAGE_CODES[0]
        if args:
            if args[0] in LANGUAGE_CODES:
                language_code = args[0]
                args.pop(0)
        
        requested_filename = os.path.join(APPDIR, *args)

        print
        print requested_filename
        print
        
        if os.path.isdir(requested_filename):
            # Handelt es sich um die index.tmpl?
            for filename in [
                os.path.join(requested_filename, "%s.index.tmpl" % language_code),
                os.path.join(requested_filename, "index.tmpl"),
            ]:
                if os.path.isfile(filename):
                    # Fehlenden Slash anhngen
                    request = cherrypy.request
                    if not request.path_info.endswith('/'):
                        new_url = cherrypy.url(
                            request.path_info + '/', request.query_string
                        )
                        raise cherrypy.HTTPRedirect(new_url)
                    break
            else:
                raise cherrypy.NotFound()
        elif requested_filename.lower().endswith(".tmpl"):
            # Benannte Vorlage
            dirname = os.path.dirname(requested_filename)
            basename = os.path.basename(requested_filename)
            if basename.startswith(language_code + "."):
                basename = basename[len(language_code) + 1:]
            for filename in [
                os.path.join(dirname, "%s.%s" % (language_code, basename)),
                os.path.join(dirname, basename),
            ]:
                if os.path.isfile(filename):
                    break
            else:
                raise cherrypy.NotFound()
        else:
            # Keine anderen Dateien ausliefern
            raise cherrypy.NotFound()
        
        # Vorlage holen, rendern und zurck geben
        template = Template(file = filename)
        template._ = gettext.gettext
        template.mtime = time.strftime(
            "%Y-%m-%d %H:%M:%S", time.localtime(os.path.getmtime(filename))
        )
        template.language_code = language_code
        return str(template).decode(TEMPLATE_CODING)
    
    default.exposed = True


def main():
    
    app = cherrypy.tree.mount(Root(), config = INI_FILENAME)
    cherrypy.config.update(
        {
            #'environment': 'production',
            #'log.screen': False,
            #'log.error_file': 'site.log',
            #'show_tracebacks': False,
            #'engine.autoreload_on': False
            "tools.staticdir.root": APPDIR,
            #"server.socket_host": "127.0.0.1",
            #"server.socket_port": 8080,
        }
    )
    cherrypy.quickstart(app)


if __name__ == "__main__":
    main()




